const outerWidth = 960;
const outerHeight = 500;
const margin = { left: 138, top: 10, right: 30, bottom: 47 };
const barPadding = 0.2;

const xColumn = "population";
const yColumn = "country";
const colorColumn = "religion";
const layerColumn = colorColumn;

let hoveredColorValue;
const hoveredStrokeColor = "black";

const innerWidth  = outerWidth  - margin.left - margin.right;
const innerHeight = outerHeight - margin.top  - margin.bottom;

const svg = d3.select("body").append("svg")
  .attr("width",  outerWidth)
  .attr("height", outerHeight);
const g = svg.append("g")
  .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

// This is the layer where the bars are drawn.
const baseBarLayer = g.append("g");

// This layer contains a semi-transparent overlay
// that fades out the base bars.
const overlayRect = g.append("g")
  .append("rect")
  .attr("width", innerWidth)
  .attr("height", innerHeight)
  .attr("fill", "none")
  .style("pointer-events", "none");

// This contains the subset of bars rendered on top
// when you hover over the entries in the color legend.
const foregroundBarLayer = g.append("g");

const xAxisG = g.append("g")
  .attr("class", "x axis")
  .attr("transform", "translate(0," + innerHeight + ")");
const yAxisG = g.append("g")
  .attr("class", "y axis");
const colorLegendG = g.append("g")
  .attr("class", "color-legend")
  .attr("transform", "translate(596, 0)");

const xScale = d3.scaleLinear().range([0, innerWidth]);
const yScale = d3.scaleBand().range([innerHeight, 0], barPadding);
const colorScale = d3.scaleOrdinal(d3.schemeCategory10);

const tipNumberFormat = d3.format(",");
const tip = d3.tip()
  .attr("class", "d3-tip")
  .offset([-10, 0])
  .html(function(d) {
    return [
      d[colorColumn],
      " in ",
      d[yColumn],
      ": ",
      tipNumberFormat(d[xColumn])
    ].join("");
  });
g.call(tip);

// Use a modified SI formatter that uses "B" for Billion.
const siFormat = d3.format("s");
const customTickFormat = function (d){
  return siFormat(d).replace("G", "B");
};

const xAxis = d3.axisBottom().scale(xScale)
  .ticks(5)
  .tickFormat(customTickFormat)
  //.outerTickSize(0);
const yAxis = d3.axisLeft().scale(yScale)
  //.outerTickSize(0);
const colorLegend = d3.legendColor()
  .scale(colorScale)
  .shapePadding(6.24)
  .shapeWidth(25)
  .shapeHeight(25)
  .labelOffset(5);

function render(data){

  const nested = d3.nest()
    .key(d => d[layerColumn])
    .entries(data);
  
  const casted = nested[0].values.map((d, i) => {
    const row = {};
    nested.forEach(({key, values}) => {
      row[key] = values[i][xColumn];
    });
    row[yColumn] = nested[0].values[i][yColumn];
    return row;
  });
  
  console.log(casted)
  
  const stack = d3.stack()
    .keys(nested.map(d => d.key));
  
  const layers = stack(casted).reverse();

//   const layers = stack(nested.reverse()).reverse();

  
  xScale.domain([
    0,
    d3.max(layers, d => d3.max(d, d => d[1]))
  ]);

  yScale.domain(data.map(d => d[yColumn]));
  colorScale.domain(data.map(d => d[colorColumn]));
  
  xAxisG.call(xAxis); 
  yAxisG.call(yAxis);

  renderBars(baseBarLayer, layers);

//   if(hoveredColorValue){
//     setOverlayTransparency(0.7);
//     renderBars(foregroundBarLayer, layers.filter(function (layer){
//       return layer.key === hoveredColorValue;
//     }));
//   } else {
//     setOverlayTransparency(0.0);
//     renderBars(foregroundBarLayer, []);
//   }

//   colorLegendG.call(colorLegend);

//   // Move the text down a bit.
//   colorLegendG.selectAll("text").attr("y", 4);

//   listenForHover(colorLegendG.selectAll("rect"), data);
//   listenForHover(colorLegendG.selectAll("text"), data);
}

function renderBars(g, layers){
  let layerGs = g.selectAll(".layer").data(layers);
  layerGs.exit().remove();
  layerGs = layerGs
    .enter().append("g")
      .attr("class", "layer")
    .merge(layerGs);

  layerGs.style("fill", d => colorScale(d.key));

  let bars = layerGs.selectAll("rect").data(d => d);
  bars.exit().remove();
  bars = bars.enter().append("rect")
    .merge(bars);
//     .on("mouseover", function(d){
//       tip.show(d);

//       // Fix the issue where the tip goes off the screen.
//       d3.select(".d3-tip").style("left", "100px");
//     })
//     .on("mouseout", tip.hide);
  
  bars
    .attr("x", d => xScale(d[1]))
    .attr("y", d => yScale(d.data[yColumn]))
    .attr("width", d => xScale(d[1]) - xScale(d[0]))
    .attr("height", yScale.bandwidth());
}

function listenForHover(selection, data){
  selection
    .on("mouseover", function (d){
      hoveredColorValue = d;
      render(data);
    })
    .on("mouseout", function (d){
      hoveredColorValue = null;
      render(data);
    })
    .style("cursor", "pointer");
}

function setOverlayTransparency(alpha){
  overlayRect
    .transition().duration(400)
    .attr("fill", "rgba(255, 255, 255, " + alpha + ")");
}

function type(d){
  d.population = +d.population;

  // Shorten this, as the long label is problematic
  if(d.country === 'Democratic Republic of the Congo'){
    d.country = 'DR Congo';
  }
  return d;
}

const dataUrl = 'https://raw.githubusercontent.com/curran/data/gh-pages/pew/religion/processed/religionByCountryTop20.csv';
d3.csv(dataUrl, type).then(render);
